"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[8921],{94055:function(a){function b(a){a.languages.graphql={comment:/#.*/,description:{pattern:/(?:"""(?:[^"]|(?!""")")*"""|"(?:\\.|[^\\"\r\n])*")(?=\s*[a-z_])/i,greedy:!0,alias:"string",inside:{"language-markdown":{pattern:/(^"(?:"")?)(?!\1)[\s\S]+(?=\1$)/,lookbehind:!0,inside:a.languages.markdown}}},string:{pattern:/"""(?:[^"]|(?!""")")*"""|"(?:\\.|[^\\"\r\n])*"/,greedy:!0},number:/(?:\B-|\b)\d+(?:\.\d+)?(?:e[+-]?\d+)?\b/i,boolean:/\b(?:false|true)\b/,variable:/\$[a-z_]\w*/i,directive:{pattern:/@[a-z_]\w*/i,alias:"function"},"attr-name":{pattern:/\b[a-z_]\w*(?=\s*(?:\((?:[^()"]|"(?:\\.|[^\\"\r\n])*")*\))?:)/i,greedy:!0},"atom-input":{pattern:/\b[A-Z]\w*Input\b/,alias:"class-name"},scalar:/\b(?:Boolean|Float|ID|Int|String)\b/,constant:/\b[A-Z][A-Z_\d]*\b/,"class-name":{pattern:/(\b(?:enum|implements|interface|on|scalar|type|union)\s+|&\s*|:\s*|\[)[A-Z_]\w*/,lookbehind:!0},fragment:{pattern:/(\bfragment\s+|\.{3}\s*(?!on\b))[a-zA-Z_]\w*/,lookbehind:!0,alias:"function"},"definition-mutation":{pattern:/(\bmutation\s+)[a-zA-Z_]\w*/,lookbehind:!0,alias:"function"},"definition-query":{pattern:/(\bquery\s+)[a-zA-Z_]\w*/,lookbehind:!0,alias:"function"},keyword:/\b(?:directive|enum|extend|fragment|implements|input|interface|mutation|on|query|repeatable|scalar|schema|subscription|type|union)\b/,operator:/[!=|&]|\.{3}/,"property-query":/\w+(?=\s*\()/,object:/\w+(?=\s*\{)/,punctuation:/[!(){}\[\]:=,]/,property:/\w+/},a.hooks.add("after-tokenize",function(a){if("graphql"===a.language){for(var b=a.tokens.filter(function(a){return"string"!=typeof a&&"comment"!==a.type&&"scalar"!==a.type}),c=0;c<b.length;){var d=b[c++];if("keyword"===d.type&&"mutation"===d.content){var e=[];if(l(["definition-mutation","punctuation"])&&"("===k(1).content){c+=2;var f=m(/^\($/,/^\)$/);if(-1===f)continue;for(;c<f;c++){var g=k(0);"variable"===g.type&&(n(g,"variable-input"),e.push(g.content))}c=f+1}if(l(["punctuation","property-query"])&&"{"===k(0).content&&(c++,n(k(0),"property-mutation"),e.length>0)){var h=m(/^\{$/,/^\}$/);if(-1===h)continue;for(var i=c;i<h;i++){var j=b[i];"variable"===j.type&&e.indexOf(j.content)>=0&&n(j,"variable-input")}}}}}function k(a){return b[c+a]}function l(a,b){b=b||0;for(var c=0;c<a.length;c++){var d=k(c+b);if(!d||d.type!==a[c])return!1}return!0}function m(a,d){for(var e=1,f=c;f<b.length;f++){var g=b[f],h=g.content;if("punctuation"===g.type&&"string"==typeof h){if(a.test(h))e++;else if(d.test(h)&&0== --e)return f}}return -1}function n(a,b){var c=a.alias;c?Array.isArray(c)||(a.alias=c=[c]):a.alias=c=[],c.push(b)}})}a.exports=b,b.displayName="graphql",b.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_graphql.15332dbd9f2d2403.js.map