self.__BUILD_MANIFEST={__rewrites:{beforeFiles:[],afterFiles:[],fallback:[]},"/":["static/chunks/pages/index-f45534c2dfd80b9f.js"],"/404":["static/chunks/pages/404-6ceccf84e6f300fe.js"],"/_error":["static/chunks/pages/_error-b04df28c0915943c.js"],"/connections":["static/chunks/pages/connections-0fc68eff45cea5a3.js"],"/query":["static/chunks/pages/query-469b79090675947c.js"],"/search":["static/chunks/pages/search-31b270dcde7e866f.js"],"/[type]":["static/chunks/pages/[type]-9400f226b78bb25f.js"],"/[type]/[entity]":["static/chunks/pages/[type]/[entity]-212c5e0002fbf5d6.js"],sortedPages:["/","/404","/_app","/_error","/connections","/query","/search","/[type]","/[type]/[entity]"]},self.__BUILD_MANIFEST_CB&&self.__BUILD_MANIFEST_CB()