# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: featureform/proto/serving.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x66\x65\x61tureform/proto/serving.proto\x12\x19\x66\x65\x61tureform.serving.proto\"\x15\n\x05Model\x12\x0c\n\x04name\x18\x01 \x01(\t\"}\n\x13TrainingDataRequest\x12\x35\n\x02id\x18\x01 \x01(\x0b\x32).featureform.serving.proto.TrainingDataID\x12/\n\x05model\x18\x02 \x01(\x0b\x32 .featureform.serving.proto.Model\"/\n\x0eTrainingDataID\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"v\n\x0fTrainingDataRow\x12\x32\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32 .featureform.serving.proto.Value\x12/\n\x05label\x18\x02 \x01(\x0b\x32 .featureform.serving.proto.Value\"\xb3\x01\n\x13\x46\x65\x61tureServeRequest\x12\x36\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32$.featureform.serving.proto.FeatureID\x12\x33\n\x08\x65ntities\x18\x02 \x03(\x0b\x32!.featureform.serving.proto.Entity\x12/\n\x05model\x18\x03 \x01(\x0b\x32 .featureform.serving.proto.Model\">\n\nFeatureRow\x12\x30\n\x06values\x18\x01 \x03(\x0b\x32 .featureform.serving.proto.Value\"*\n\tFeatureID\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"%\n\x06\x45ntity\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\x8a\x02\n\x05Value\x12\x13\n\tstr_value\x18\x01 \x01(\tH\x00\x12\x13\n\tint_value\x18\x02 \x01(\x05H\x00\x12\x15\n\x0b\x66loat_value\x18\x03 \x01(\x02H\x00\x12\x16\n\x0c\x64ouble_value\x18\x04 \x01(\x01H\x00\x12\x15\n\x0bint64_value\x18\x05 \x01(\x03H\x00\x12\x15\n\x0bint32_value\x18\x06 \x01(\x05H\x00\x12\x14\n\nbool_value\x18\x07 \x01(\x08H\x00\x12\x1c\n\x12on_demand_function\x18\x08 \x01(\x0cH\x00\x12=\n\x0evector32_value\x18\t \x01(\x0b\x32#.featureform.serving.proto.Vector32H\x00\x42\x07\n\x05value\")\n\x08SourceID\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"S\n\x11SourceDataRequest\x12/\n\x02id\x18\x01 \x01(\x0b\x32#.featureform.serving.proto.SourceID\x12\r\n\x05limit\x18\x02 \x01(\x03\"F\n\x13SourceColumnRequest\x12/\n\x02id\x18\x01 \x01(\x0b\x32#.featureform.serving.proto.SourceID\"?\n\rSourceDataRow\x12.\n\x04rows\x18\x01 \x03(\x0b\x32 .featureform.serving.proto.Value\"$\n\x11SourceDataColumns\x12\x0f\n\x07\x63olumns\x18\x01 \x03(\t\"S\n\x1aTrainingDataColumnsRequest\x12\x35\n\x02id\x18\x01 \x01(\x0b\x32).featureform.serving.proto.TrainingDataID\"2\n\x0fTrainingColumns\x12\x10\n\x08\x66\x65\x61tures\x18\x01 \x03(\t\x12\r\n\x05label\x18\x02 \x01(\t\"\x19\n\x08Vector32\x12\r\n\x05value\x18\x01 \x03(\x02\"\x82\x01\n\x0eNearestRequest\x12\x30\n\x02id\x18\x01 \x01(\x0b\x32$.featureform.serving.proto.FeatureID\x12\x33\n\x06vector\x18\x02 \x01(\x0b\x32#.featureform.serving.proto.Vector32\x12\t\n\x01k\x18\x03 \x01(\x05\"#\n\x0fNearestResponse\x12\x10\n\x08\x65ntities\x18\x01 \x03(\t2\x9d\x05\n\x07\x46\x65\x61ture\x12n\n\x0cTrainingData\x12..featureform.serving.proto.TrainingDataRequest\x1a*.featureform.serving.proto.TrainingDataRow\"\x00\x30\x01\x12z\n\x13TrainingDataColumns\x12\x35.featureform.serving.proto.TrainingDataColumnsRequest\x1a*.featureform.serving.proto.TrainingColumns\"\x00\x12g\n\x0c\x46\x65\x61tureServe\x12..featureform.serving.proto.FeatureServeRequest\x1a%.featureform.serving.proto.FeatureRow\"\x00\x12h\n\nSourceData\x12,.featureform.serving.proto.SourceDataRequest\x1a(.featureform.serving.proto.SourceDataRow\"\x00\x30\x01\x12o\n\rSourceColumns\x12..featureform.serving.proto.SourceColumnRequest\x1a,.featureform.serving.proto.SourceDataColumns\"\x00\x12\x62\n\x07Nearest\x12).featureform.serving.proto.NearestRequest\x1a*.featureform.serving.proto.NearestResponse\"\x00\x42\x1eZ\x1cgithub.com/featureform/protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'featureform.proto.serving_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\034github.com/featureform/proto'
  _globals['_MODEL']._serialized_start=62
  _globals['_MODEL']._serialized_end=83
  _globals['_TRAININGDATAREQUEST']._serialized_start=85
  _globals['_TRAININGDATAREQUEST']._serialized_end=210
  _globals['_TRAININGDATAID']._serialized_start=212
  _globals['_TRAININGDATAID']._serialized_end=259
  _globals['_TRAININGDATAROW']._serialized_start=261
  _globals['_TRAININGDATAROW']._serialized_end=379
  _globals['_FEATURESERVEREQUEST']._serialized_start=382
  _globals['_FEATURESERVEREQUEST']._serialized_end=561
  _globals['_FEATUREROW']._serialized_start=563
  _globals['_FEATUREROW']._serialized_end=625
  _globals['_FEATUREID']._serialized_start=627
  _globals['_FEATUREID']._serialized_end=669
  _globals['_ENTITY']._serialized_start=671
  _globals['_ENTITY']._serialized_end=708
  _globals['_VALUE']._serialized_start=711
  _globals['_VALUE']._serialized_end=977
  _globals['_SOURCEID']._serialized_start=979
  _globals['_SOURCEID']._serialized_end=1020
  _globals['_SOURCEDATAREQUEST']._serialized_start=1022
  _globals['_SOURCEDATAREQUEST']._serialized_end=1105
  _globals['_SOURCECOLUMNREQUEST']._serialized_start=1107
  _globals['_SOURCECOLUMNREQUEST']._serialized_end=1177
  _globals['_SOURCEDATAROW']._serialized_start=1179
  _globals['_SOURCEDATAROW']._serialized_end=1242
  _globals['_SOURCEDATACOLUMNS']._serialized_start=1244
  _globals['_SOURCEDATACOLUMNS']._serialized_end=1280
  _globals['_TRAININGDATACOLUMNSREQUEST']._serialized_start=1282
  _globals['_TRAININGDATACOLUMNSREQUEST']._serialized_end=1365
  _globals['_TRAININGCOLUMNS']._serialized_start=1367
  _globals['_TRAININGCOLUMNS']._serialized_end=1417
  _globals['_VECTOR32']._serialized_start=1419
  _globals['_VECTOR32']._serialized_end=1444
  _globals['_NEARESTREQUEST']._serialized_start=1447
  _globals['_NEARESTREQUEST']._serialized_end=1577
  _globals['_NEARESTRESPONSE']._serialized_start=1579
  _globals['_NEARESTRESPONSE']._serialized_end=1614
  _globals['_FEATURE']._serialized_start=1617
  _globals['_FEATURE']._serialized_end=2286
# @@protoc_insertion_point(module_scope)
