"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[5755],{5261:function(a){function b(a){!function(a){var b={pattern:/(^[ \t]*| {2}|\t)#.*/m,lookbehind:!0,greedy:!0},c={pattern:/((?:^|[^\\])(?:\\{2})*)[$@&%]\{(?:[^{}\r\n]|\{[^{}\r\n]*\})*\}/,lookbehind:!0,inside:{punctuation:/^[$@&%]\{|\}$/}};function d(a,d){var e={};for(var f in e["section-header"]={pattern:/^ ?\*{3}.+?\*{3}/,alias:"keyword"},d)e[f]=d[f];return e.tag={pattern:/([\r\n](?: {2}|\t)[ \t]*)\[[-\w]+\]/,lookbehind:!0,inside:{punctuation:/\[|\]/}},e.variable=c,e.comment=b,{pattern:RegExp(/^ ?\*{3}[ \t]*<name>[ \t]*\*{3}(?:.|[\r\n](?!\*{3}))*/.source.replace(/<name>/g,function(){return a}),"im"),alias:"section",inside:e}}var e={pattern:/(\[Documentation\](?: {2}|\t)[ \t]*)(?![ \t]|#)(?:.|(?:\r\n?|\n)[ \t]*\.{3})+/,lookbehind:!0,alias:"string"},f={pattern:/([\r\n] ?)(?!#)(?:\S(?:[ \t]\S)*)+/,lookbehind:!0,alias:"function",inside:{variable:c}},g={pattern:/([\r\n](?: {2}|\t)[ \t]*)(?!\[|\.{3}|#)(?:\S(?:[ \t]\S)*)+/,lookbehind:!0,inside:{variable:c}};a.languages.robotframework={settings:d("Settings",{documentation:{pattern:/([\r\n] ?Documentation(?: {2}|\t)[ \t]*)(?![ \t]|#)(?:.|(?:\r\n?|\n)[ \t]*\.{3})+/,lookbehind:!0,alias:"string"},property:{pattern:/([\r\n] ?)(?!\.{3}|#)(?:\S(?:[ \t]\S)*)+/,lookbehind:!0}}),variables:d("Variables"),"test-cases":d("Test Cases",{"test-name":f,documentation:e,property:g}),keywords:d("Keywords",{"keyword-name":f,documentation:e,property:g}),tasks:d("Tasks",{"task-name":f,documentation:e,property:g}),comment:b},a.languages.robot=a.languages.robotframework}(a)}a.exports=b,b.displayName="robotframework",b.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_robotframework.438dde1044b048c2.js.map