"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[6749],{76942:function(a){function b(a){a.languages.hpkp={directive:{pattern:/\b(?:includeSubDomains|max-age|pin-sha256|preload|report-to|report-uri|strict)(?=[\s;=]|$)/i,alias:"property"},operator:/=/,punctuation:/;/}}a.exports=b,b.displayName="hpkp",b.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_hpkp.644eb598362ed3c4.js.map