"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[7393],{96319:function(a){function b(a){a.languages.yang={comment:/\/\*[\s\S]*?\*\/|\/\/.*/,string:{pattern:/"(?:[^\\"]|\\.)*"|'[^']*'/,greedy:!0},keyword:{pattern:/(^|[{};\r\n][ \t]*)[a-z_][\w.-]*/i,lookbehind:!0},namespace:{pattern:/(\s)[a-z_][\w.-]*(?=:)/i,lookbehind:!0},boolean:/\b(?:false|true)\b/,operator:/\+/,punctuation:/[{};:]/}}a.exports=b,b.displayName="yang",b.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_yang.940debc626008644.js.map