"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[2509,4657,9461],{96412:function(a){function b(a){!function(a){var b=a.util.clone(a.languages.javascript),c=/(?:\s|\/\/.*(?!.)|\/\*(?:[^*]|\*(?!\/))\*\/)/.source,d=/(?:\{(?:\{(?:\{[^{}]*\}|[^{}])*\}|[^{}])*\})/.source,e=/(?:\{<S>*\.{3}(?:[^{}]|<BRACES>)*\})/.source;function f(a,b){return RegExp(a=a.replace(/<S>/g,function(){return c}).replace(/<BRACES>/g,function(){return d}).replace(/<SPREAD>/g,function(){return e}),b)}e=f(e).source,a.languages.jsx=a.languages.extend("markup",b),a.languages.jsx.tag.pattern=f(/<\/?(?:[\w.:-]+(?:<S>+(?:[\w.:$-]+(?:=(?:"(?:\\[\s\S]|[^\\"])*"|'(?:\\[\s\S]|[^\\'])*'|[^\s{'"/>=]+|<BRACES>))?|<SPREAD>))*<S>*\/?)?>/.source),a.languages.jsx.tag.inside.tag.pattern=/^<\/?[^\s>\/]*/,a.languages.jsx.tag.inside["attr-value"].pattern=/=(?!\{)(?:"(?:\\[\s\S]|[^\\"])*"|'(?:\\[\s\S]|[^\\'])*'|[^\s'">]+)/,a.languages.jsx.tag.inside.tag.inside["class-name"]=/^[A-Z]\w*(?:\.[A-Z]\w*)*$/,a.languages.jsx.tag.inside.comment=b.comment,a.languages.insertBefore("inside","attr-name",{spread:{pattern:f(/<SPREAD>/.source),inside:a.languages.jsx}},a.languages.jsx.tag),a.languages.insertBefore("inside","special-attr",{script:{pattern:f(/=<BRACES>/.source),alias:"language-javascript",inside:{"script-punctuation":{pattern:/^=(?=\{)/,alias:"punctuation"},rest:a.languages.jsx}}},a.languages.jsx.tag);var g=function(a){return a?"string"==typeof a?a:"string"==typeof a.content?a.content:a.content.map(g).join(""):""},h=function(b){for(var c=[],d=0;d<b.length;d++){var e=b[d],f=!1;if("string"!=typeof e&&("tag"===e.type&&e.content[0]&&"tag"===e.content[0].type?"</"===e.content[0].content[0].content?c.length>0&&c[c.length-1].tagName===g(e.content[0].content[1])&&c.pop():"/>"===e.content[e.content.length-1].content||c.push({tagName:g(e.content[0].content[1]),openedBraces:0}):c.length>0&&"punctuation"===e.type&&"{"===e.content?c[c.length-1].openedBraces++:c.length>0&&c[c.length-1].openedBraces>0&&"punctuation"===e.type&&"}"===e.content?c[c.length-1].openedBraces--:f=!0),(f||"string"==typeof e)&&c.length>0&&0===c[c.length-1].openedBraces){var i=g(e);d<b.length-1&&("string"==typeof b[d+1]||"plain-text"===b[d+1].type)&&(i+=g(b[d+1]),b.splice(d+1,1)),d>0&&("string"==typeof b[d-1]||"plain-text"===b[d-1].type)&&(i=g(b[d-1])+i,b.splice(d-1,1),d--),b[d]=new a.Token("plain-text",i,null,i)}e.content&&"string"!=typeof e.content&&h(e.content)}};a.hooks.add("after-tokenize",function(a){("jsx"===a.language||"tsx"===a.language)&&h(a.tokens)})}(a)}a.exports=b,b.displayName="jsx",b.aliases=[]},87041:function(a,b,c){var d=c(96412),e=c(4979);function f(a){var b,c,f;a.register(d),a.register(e),c=(b=a).util.clone(b.languages.typescript),b.languages.tsx=b.languages.extend("jsx",c),delete b.languages.tsx.parameter,delete b.languages.tsx["literal-property"],f=b.languages.tsx.tag,f.pattern=RegExp(/(^|[^\w$]|(?=<\/))/.source+"(?:"+f.pattern.source+")",f.pattern.flags),f.lookbehind=!0}a.exports=f,f.displayName="tsx",f.aliases=[]},4979:function(a){function b(a){var b,c;(b=a).languages.typescript=b.languages.extend("javascript",{"class-name":{pattern:/(\b(?:class|extends|implements|instanceof|interface|new|type)\s+)(?!keyof\b)(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?:\s*<(?:[^<>]|<(?:[^<>]|<[^<>]*>)*>)*>)?/,lookbehind:!0,greedy:!0,inside:null},builtin:/\b(?:Array|Function|Promise|any|boolean|console|never|number|string|symbol|unknown)\b/}),b.languages.typescript.keyword.push(/\b(?:abstract|declare|is|keyof|readonly|require)\b/,/\b(?:asserts|infer|interface|module|namespace|type)\b(?=\s*(?:[{_$a-zA-Z\xA0-\uFFFF]|$))/,/\btype\b(?=\s*(?:[\{*]|$))/),delete b.languages.typescript.parameter,delete b.languages.typescript["literal-property"],delete(c=b.languages.extend("typescript",{}))["class-name"],b.languages.typescript["class-name"].inside=c,b.languages.insertBefore("typescript","function",{decorator:{pattern:/@[$\w\xA0-\uFFFF]+/,inside:{at:{pattern:/^@/,alias:"operator"},function:/^[\s\S]+/}},"generic-function":{pattern:/#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*\s*<(?:[^<>]|<(?:[^<>]|<[^<>]*>)*>)*>(?=\s*\()/,greedy:!0,inside:{function:/^#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*/,generic:{pattern:/<[\s\S]+/,alias:"class-name",inside:c}}}}),b.languages.ts=b.languages.typescript}a.exports=b,b.displayName="typescript",b.aliases=["ts"]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_tsx.996a68f2e8e7b36b.js.map