"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[2180],{7065:function(a,b,c){var d=c(45950);function e(a){var b,c;a.register(d),c=/("|')(?:\\(?:\r\n?|\n|.)|(?!\1)[^\\\r\n])*\1/,(b=a).languages.json5=b.languages.extend("json",{property:[{pattern:RegExp(c.source+"(?=\\s*:)"),greedy:!0},{pattern:/(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*:)/,alias:"unquoted"}],string:{pattern:c,greedy:!0},number:/[+-]?\b(?:NaN|Infinity|0x[a-fA-F\d]+)\b|[+-]?(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[eE][+-]?\d+\b)?/})}a.exports=e,e.displayName="json5",e.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_json5.40733c54ed05194e.js.map