"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[902],{9858:function(a){function b(a){var b,c;Object.defineProperty(c=(b=a).languages.javadoclike={parameter:{pattern:/(^[\t ]*(?:\/{3}|\*|\/\*\*)\s*@(?:arg|arguments|param)\s+)\w+/m,lookbehind:!0},keyword:{pattern:/(^[\t ]*(?:\/{3}|\*|\/\*\*)\s*|\{)@[a-z][a-zA-Z-]+\b/m,lookbehind:!0},punctuation:/[{}]/},"addSupport",{value:function(a,c){"string"==typeof a&&(a=[a]),a.forEach(function(a){!function(a,c){var d="doc-comment",e=b.languages[a];if(e){var f=e[d];if(!f){var g={};g[d]={pattern:/(^|[^\\])\/\*\*[^/][\s\S]*?(?:\*\/|$)/,lookbehind:!0,alias:"comment"},f=(e=b.languages.insertBefore(a,"comment",g))[d]}if(f instanceof RegExp&&(f=e[d]={pattern:f}),Array.isArray(f))for(var h=0,i=f.length;h<i;h++)f[h]instanceof RegExp&&(f[h]={pattern:f[h]}),c(f[h]);else c(f)}}(a,function(a){a.inside||(a.inside={}),a.inside.rest=c})})}}),c.addSupport(["java","javascript","php"],c)}a.exports=b,b.displayName="javadoclike",b.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_javadoclike.6ca963116fa85528.js.map