"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[3140],{60561:function(a){function b(a){a.languages.hsts={directive:{pattern:/\b(?:includeSubDomains|max-age|preload)(?=[\s;=]|$)/i,alias:"property"},operator:/=/,punctuation:/;/}}a.exports=b,b.displayName="hsts",b.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_hsts.4286c05ccf1cd9d9.js.map