"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[7250],{62260:function(a){function b(a){a.languages.bbcode={tag:{pattern:/\[\/?[^\s=\]]+(?:\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'"\]=]+))?(?:\s+[^\s=\]]+\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'"\]=]+))*\s*\]/,inside:{tag:{pattern:/^\[\/?[^\s=\]]+/,inside:{punctuation:/^\[\/?/}},"attr-value":{pattern:/=\s*(?:"[^"]*"|'[^']*'|[^\s'"\]=]+)/,inside:{punctuation:[/^=/,{pattern:/^(\s*)["']|["']$/,lookbehind:!0}]}},punctuation:/\]/,"attr-name":/[^\s=\]]+/}}},a.languages.shortcode=a.languages.bbcode}a.exports=b,b.displayName="bbcode",b.aliases=["shortcode"]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_bbcode.e06f7befa19938d2.js.map