"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[2564],{13600:function(a){function b(a){a.languages.git={comment:/^#.*/m,deleted:/^[-–].*/m,inserted:/^\+.*/m,string:/("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,command:{pattern:/^.*\$ git .*$/m,inside:{parameter:/\s--?\w+/}},coord:/^@@.*@@$/m,"commit-sha1":/^commit \w{40}$/m}}a.exports=b,b.displayName="git",b.aliases=[]}}])
//# sourceMappingURL=react-syntax-highlighter_languages_refractor_git.8e33dec4fdbc60a6.js.map